"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.filterDiffJson = filterDiffJson;
exports.filterFileBasedOnLint = filterFileBasedOnLint;
exports.filterLintReportBasedOnImpactLine = filterLintReportBasedOnImpactLine;
var _path = _interopRequireDefault(require("path"));
var _getModifiedLines = require("../handlers/getModifiedLines");
var _getFileUtils = require("./getFileUtils");
// import { checkIfFileExists } from "../../utils/fileUtils";

/**
 * @function  filterLintReportBasedOnImpactLine -  Filter Report Based on Impact Line
 * @param {JSON} lintReport - Lint report  generated by eslint
 * @param {Map} reportLineMap - This Map contains file as key  and impacted lines as value
 * @param {string} filePath - File path of violated code
 * @returns {JSON}
 *
 * @note - If they call impact based as false then filter not apply
 */

function filterLintReportBasedOnImpactLine(lintReport, reportLineMap, filePath) {
  const impactedLines = reportLineMap.get(filePath);
  const regex = /Definition for rule '(.+)' was not found\./;
  lintReport.messages = lintReport.messages.filter(message => {
    if (filterNotDefinedRule(message.message, regex)) {
      return false;
    }
    if (impactedLines.includes(message.line)) {
      // To set pipeline status
      if (message["severity"] == 2) {
        global.analytics.status = "FAILURE";
        global.analytics.message = 'Issues are detected, please review and resolve the errors';
      }
      return true;
    }
    return false;
  });
  return lintReport;
}

/**
 * @function filterDiffJson  - Filter file path From git diff json and Create Map diff file as key  and impacted lines as value
 * @param {JSON} diffJson -  Git Diff JSON  (This JSON get by git diff api)
 * @returns {Map}
 */

function filterDiffJson(diffJson) {
  var _diffJson$diffs;
  const diffFilesMap = new Map();
  diffJson === null || diffJson === void 0 || (_diffJson$diffs = diffJson.diffs) === null || _diffJson$diffs === void 0 || _diffJson$diffs.forEach(diff => {
    const diffPathKey = _path.default.resolve((0, _getFileUtils.getRootGitPath)(), diff.new_path);
    if (diffPathKey === undefined) {
      return;
    }
    if (diffPathKey) {
      diffFilesMap.set(diffPathKey, (0, _getModifiedLines.getModifiedLines)(diff.diff).modifiedLines);
    }
  });
  return diffFilesMap;
}
function filterFileBasedOnLint(lintReport) {
  return lintReport;
}

/**
 * @function filterNotDefinedRule -  It test message with given regex
 * @param {string} message  - report message
 * @returns {Boolean}
 */

function filterNotDefinedRule(message, regex) {
  return regex.test(message);
}